##############################################
#
# cell
# Module | Main component of a Data Visualization Model like Articles or FileCards.
#
# Daniel Bakas Amuchastegui
# September 3, 2019
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from semantyk.core.model import Model

class Cell(Model):
    pass


##############################################
#
# venn_diagram
# Module | Diagram that displays all possible logical objects of type Relation between a finite collection of different objects of type Resource.
#
# Daniel Bakas Amuchastegui
# September 3, 2019
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from semantyk.data_visualization_models.graph.graph import Graph

class VennDiagram(Graph):
    pass


##############################################
#
# data_visualization_models
# Package | Interactive & Visual Models for Data Representation Package.
#
# Daniel Bakas Amuchastegui
# September 3, 2019
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from . import article
from . import cell
from . import file
from . import graph
from . import table

from .article import *
from .cell import *
from .file import *
from .graph import *
from .table import *


##############################################
#
# dvm_extract_cell
# Module | Data Visualization Model Extract Cell Module.
#
# Daniel Bakas Amuchastegui
# February, 2020
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from semantyk.data_visualization_models.cell.cell import Cell

class DVMExtractCell(Cell):
    pass


##############################################
#
# namespace 
# Package | Semantyk Namespace Module.
#
# Author: Daniel Bakas Amuchastegui
# August 31, 2017
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from rdflib import Namespace

DC = Namespace('http://purl.org/dc/elements/1.1/')
DCTERMS = Namespace('http://purl.org/dc/terms/')
DOAP = Namespace('http://usefulinc.com/ns/doap#')
FOAF = Namespace('http://xmlns.com/foaf/0.1/')
OWL = Namespace('http://www.w3.org/2002/07/owl#')
RDF = Namespace('http://www.w3.org/1999/02/22-rdf-syntax-ns#')
RDFS = Namespace('http://www.w3.org/2000/01/rdf-schema#')
SMTK = Namespace('https://raw.githubusercontent.com/semantykcom/Semantyk/master/Archive/archive.ttl/')
SKOS = Namespace('http://www.w3.org/2004/02/skos/core#')
XSD = Namespace('http://www.w3.org/2001/XMLSchema#')
VOID = Namespace('http://rdfs.org/ns/void#')


##############################################
#
# Store
# Package | Intelligent System For Product Selection Package.
#
# Daniel Bakas Amuchastegui
# March 16, 2019
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################


##############################################
#
# cell
# Package | Package containing the main components of a cell.
#
# Daniel Bakas Amuchastegui
# February, 2020
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from . import cell
from . import dvm_extract_cell
from . import multimedia_cell
from . import text_cell
from .cell import *


##############################################
#
# semantyk 
# Package | Semantyk Package.
#
# Author: Daniel Bakas Amuchastegui
# August 31, 2017
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from . import archive
from . import av_reality
from . import das
from . import data_visualization_models
from . import dots
from . import games
from . import learn
from . import namespace
from . import sam
from . import store
from . import tools
from . import uuid_generator

from .archive import *
from .av_reality import *
from .data_visualization_models import *
from .dots import *
from .games import *
from .learn import *
from .namespace import *
from .sam import *
from .store import *
from .tools import *
from .uuid_generator import *


##############################################
#
# namespace
# Package | Namespace Package.
#
# Daniel Bakas Amuchastegui
# Nay 10, 2020
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from . import namespace

from .namespace import *


##############################################
#
# Graph
# Module | Module Graph.
#
# Daniel Bakas Amuchastegui
# March 16, 2020
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from semantyk.core.resource import Resource
from semantyk.uuid_generator import generate_uuid
from semantyk.namespace import RDFS, SMTK
from rdflib import Literal, URIRef

class Graph(Resource):
    id:str
    triples:set
    uri:URIRef

    def add_triple(self, s, p:URIRef, o):
        self.triples.add((s,p,o))

    def add_triples(self, *triples:tuple):
        for triple in triples:
            self.triples.add(triple)
        
    def add_triples(self, triples:set):
        for triple in triples:
            self.triples.add(triple)

    def get(self, s = None, p:URIRef = None, o = None):
        for _s,_p,_o in self.triples:
            if (not(s) and not(p) and not(o)) or \
            (s and _s == s) or \
            (p and _p == p) or \
            (o and _o == o):
                yield (_s,_p,_o)

    def get_objects(self, s = None, p:URIRef = None, o = None):
        for _s,_p,_o in self.triples:
            if (not(s) and not(p) and not(o)) or \
            (s and _s == s) or \
            (p and _p == p) or \
            (o and _o == o):
                yield _o

    def get_object_predicates(self, s = None, p:URIRef = None, o = None):
        for _s,_p,_o in self.triples:
            if (not(s) and not(p) and not(o)) or \
            (s and _s == s) or \
            (p and _p == p) or \
            (o and _o == o):
                yield (_o,_p)

    def get_object_subjects(self, s = None, p:URIRef = None, o = None):
        for _s,_p,_o in self.triples:
            if (not(s) and not(p) and not(o)) or \
            (s and _s == s) or \
            (p and _p == p) or \
            (o and _o == o):
                yield (_o,_s)

    def get_predicates(self, s = None, p:URIRef = None, o = None):
        for _s,_p,_o in self.triples:
            if (not(s) and not(p) and not(o)) or \
            (s and _s == s) or \
            (p and _p == p) or \
            (o and _o == o):
                yield _p

    def get_predicate_objects(self, s = None, p:URIRef = None, o = None):
        for _s,_p,_o in self.triples:
            if (not(s) and not(p) and not(o)) or \
            (s and _s == s) or \
            (p and _p == p) or \
            (o and _o == o):
                yield (_p,_o)

    def get_predicate_subjects(self, s = None, p:URIRef = None, o = None):
        for _s,_p,_o in self.triples:
            if (not(s) and not(p) and not(o)) or \
            (s and _s == s) or \
            (p and _p == p) or \
            (o and _o == o):
                yield (_p,_s)

    def get_subjects(self, s = None, p:URIRef = None, o = None):
        for _s,_p,_o in self.triples:
            if (not(s) and not(p) and not(o)) or \
            (s and _s == s) or \
            (p and _p == p) or \
            (o and _o == o):
                yield _s

    def get_subject_objects(self, s = None, p:URIRef = None, o = None):
        for _s,_p,_o in self.triples:
            if (not(s) and not(p) and not(o)) or \
            (s and _s == s) or \
            (p and _p == p) or \
            (o and _o == o):
                yield (s,o)

    def get_subject_predicates(self, s = None, p:URIRef = None, o = None):
        for _s,_p,_o in self.triples:
            if (not(s) and not(p) and not(o)) or \
            (s and _s == s) or \
            (p and _p == p) or \
            (o and _o == o):
                yield (_s,_p)
    
    def linked(self, s = None, p:URIRef = None, o = None):
        triples = set()
        for _s,_p,_o in self.triples:
            if isinstance(_o, URIRef) and \
            (not(s) and not(p) and not(o)) or \
            (s and _s == s) or \
            (p and _p == p) or \
            (o and _o == o):
                triples.add((_s,_p,_o))
        return Graph(self.id, triples = triples)

    def mirror(self, s = None, p:URIRef = None, o = None):
        for _s,_p,_o in self.triples:
            if (not(s) and not(p) and not(o)) or \
            (s and _s == s) or \
            (p and _p == p) or \
            (o and _o == o):
                yield (_o,_p,_s)

    def qualitative(self, s = None, p:URIRef = None, o = None, linked:bool = False):
        triples = set()
        for _s,_p,_o in self.triples:
            if isinstance(_o, Literal) and \
            (isinstance(_o.value, bool) or not(isinstance(_o.value, (int, float)))) and \
            (not(s) and not(p) and not(o)) or \
            (s and _s == s) or \
            (p and _p == p) or \
            (o and _o == o):
                triples.add((s,p,o.value))
            if linked and isinstance(o, URIRef):
                triples.add((s,p,o.value))
        return Graph(self.id, triples = triples)

    def quantitative(self, s = None, p:URIRef = None, o = None, linked:bool = False):
        triples = set()
        for _s,_p,_o in self.triples:
            if (isinstance(o, Literal) and \
            isinstance(o.value, (int, float)) and \
            not(isinstance(o.value, bool))) and \
            (not(s) and not(p) and not(o)) or \
            (s and _s == s) or \
            (p and _p == p) or \
            (o and _o == o):
                triples.add((s,p,o.value))
            if linked and isinstance(o, URIRef):
                triples.add((s,p,o.value))
        return Graph(self.id, triples = triples)

    def remove_triple(self, s = None, p:URIRef = None, o = None):
        for _s,_p,_o in self.triples:
            if (not(s) and not(p) and not(o)) or \
            (s and _s == s) or \
            (p and _p == p) or \
            (o and _o == o):
                self.triples.remove((_s,_p,_o))

    def remove_triples(self, triples:set):
        for triple in triples:
            self.triples.remove(triple)

    def to_dict(self):
        dictionary = dict()
        for s,p,o in self.triples:
            if str(s) not in dictionary:
                dictionary[str(s)] = dict()
            if str(p) not in dictionary[str(s)]:
                dictionary[str(s)][str(p)] = set()
            dictionary[str(s)][str(p)].add(o)
        return dictionary

    def to_resources(self, s = None, p:URIRef = None, o = None):
        resources = set()
        for _s,_p,_o in self.triples:
            if (not(s) and not(p) and not(o)) or \
            (s and _s == s) or \
            (p and _p == p) or \
            (o and _o == o):
                if not resources:resources.add(Resource(dictionary = {str(_p): {_o}}, uri = _s))
                for resource in resources:
                    if _s == resource.uri:
                        resource.add_value(str(_p), _o)
        return resources
    resources = property(to_resources)

    def update_triples(self, new_triple:tuple, old_triple:tuple):
        self.triples.remove(old_triple)
        self.triples.add(new_triple)

    def __eq__(self, other):
        return super().__eq__(other) and self.triples == other.triples

    def __init__(self, id:str = None, dictionary:dict = None, triples:set = None, uri:URIRef = None):
        super().__init__(id, dictionary, uri = uri)
        self.triples = triples

    def __iter__(self):
        for triple in self.triples:
            yield triple

    def __str__(self):
        result = str(self.uri)
        if self.triples:
            result += ':'
            triples = self.to_dict()
            for s in triples:
                result += '\n - ' + str(s) + ':'
                for p in triples[s]:
                    result += '\n  - ' + str(p)
                    for o in triples[s][p]:
                        result += '\n   - ' + str(o)
        return result


##############################################
#
# resource 
# Module | Resource Module.
#
# Author: Daniel Bakas Amuchastegui
# March 18, 2020
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from semantyk.namespace.namespace import RDF, RDFS, SMTK
from semantyk.uuid_generator import generate_uuid 
from rdflib import Namespace, URIRef

class Resource(object):
    dictionary:dict
    id:str
    uri:URIRef

    def _get_isDefinedBy(self):
        return self.get_values(str(RDFS.isDefinedBy))
    isDefinedBy = property(_get_isDefinedBy)

    def _get_label(self):
        return self.get_values(str(RDFS.label))
    label = property(_get_label)

    def _get_type(self):
        return self.get_values(str(RDF.type))
    type = property(_get_type)

    def _get_properties(self):
        for p in self.dictionary: yield p
    properties = property(_get_properties)

    def _get_subClassOf(self):
        return self.get_values(str(RDFS.subClassOf))
    subClassOf = property(_get_subClassOf)

    def _get_values(self):
        return self.get_values()
    values = property(_get_values)

    def add_property(self, p:str):
        if p not in self.dictionary: self.dictionary[p] = set()

    def add_value(self, p:str, o):
        self.add_property(p)
        self.dictionary[p].add(o)

    def delete_property(self, p:str):
        if p in self.dictionary: del self.dictionary[p]

    def delete_value(self, p:str, o):
        if p in self.dictionary: self.dictionary[p].remove(o)

    def from_triples(self, triples:set = None):
        dictionary = dict()
        if triples:
            for triple in triples:
                if str(triple[1]) not in dictionary:
                    dictionary[str(triple[1])] = set()
                dictionary[str(triple[1])].add(triple[2])
        return dictionary
            
    def get_values(self, p:str):
        if p in self.dictionary:
            for o in self.dictionary[p]: yield o

    def to_triples(self):
        triples = set()
        for p in self.dictionary:
            for o in self.dictionary[p]:
                triples.add((self.uri, URIRef(p), o))
        return triples

    def update_property(self, old_p:str, new_p:str):
        self.dictionary[new_p] = self.dictionary[old_p]
        self.delete_property(old_p)

    def update_value(self, p:str, old_o, new_o):
        self.delete_value(p, old_o)
        self.add_value(p, new_o)

    def _generate_id(self, uri:URIRef = None):
        if uri and self.dictionary and str(RDFS.isDefinedBy) in self.dictionary:
            return str(uri).replace(list(self.dictionary[str(RDFS.isDefinedBy)])[0], '')
        return generate_uuid()

    def _generate_uri(self):
        if self.dictionary and str(RDFS.isDefinedBy) in self.dictionary:
            result = URIRef(str(next(self.isDefinedBy)) + self.id)
        return URIRef(SMTK) + self.id or self._generate_id()

    def __eq__(self, other):
        return self.__class__ == other.__class__ and self.dictionary == other.dictionary and self.id == other.id and self.uri == other.uri

    def __init__(self, id:str = None, dictionary:dict = None, triples:set = None, uri:URIRef = None):
        self.dictionary = dictionary or self.from_triples(triples)
        self.id = id or self._generate_id(uri)
        self.uri = uri or self._generate_uri()

    def __iter__(self):
        return self.properties

    def __hash__(self):
        return hash(self.uri)

    def __str__(self):
        result = self.id
        if self.dictionary:
            result += ':'
            for p in self.dictionary:
                result += '\n - ' + str(p)
                if self.dictionary[p]:
                    result += ':'
                    for o in self.dictionary[p]:
                        result += '\n  - ' + str(o)
        return result


##############################################
#
# graph
# Package | graph Package.
#
# Daniel Bakas Amuchastegui
# March 16, 2020
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from . import chart
from . import graph
from . import network
from . import timeline
from . import venn_diagram

from .chart import *
from .graph import *
from .network import *
from .timeline import *
from .venn_diagram import *


##############################################
#
# article
# Module | Written compositions describing the content of a Resouce, through the use of Cells.
#
# Daniel Bakas Amuchastegui
# September 3, 2019
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from semantyk.core.model import Model

class Article(Model):
    pass